@@ -1,3 +1,4 @@
+from sre_parse import CATEGORIES
 from textwrap import wrap
 from nebula.graph import ttypes,GraphService
 from nebula.ConnectionPool import ConnectionPool
@@ -454,6 +455,134 @@ def ucreateEdge(gClient,edgeAttrTypeDefaultDict,defaultEdgeAttrTypeDict={"alias"
         attrListStr=",".join(attrList)
         gClient.execute_query("CREATE EDGE IF NOT EXISTS {} ({})".format(tagTypeItem,attrListStr))
 
+def uploadGraph(gClient,folderName,cookie,
+                    tgtGdbUser,tgtGdbPassword,tgtGdbHost,tgtGdbPort,
+                    tgtGOfficialAPI):
+    '''上传图数据库'''
+    uploadSchemaJson["force"]=uploadSchemaJson.get("force",False)
+    maintain_csv2platodb_A_uploadFile.buildGraphDB(gClient,
+                                                    uploadSchemaJson,
+                                                    realDbName=uploadSchemaJson["gDbName"],
+                                                    force=uploadSchemaJson.get("force",False))
+    useSpaceStr="USE {}".format(uploadSchemaJson["gDbName"])
+    gClient.execute(useSpaceStr)
+    gClient.set_space(uploadSchemaJson["gDbName"])
+    uploadSchemaJson.pop("force")
+    uploadSchemaJson=maintain_csv2platodb_A_uploadFile.remakeRawSchema(uploadSchemaJson,csvFolder=folderName,gClient=gClient)
+    with open(folderName+"/uploadSchema.json","w+",encoding="utf8") as uploadSchemaJsonFile:
+        json.dump(uploadSchemaJson,uploadSchemaJsonFile)
+    
+    print("uploading ...",tgtGOfficialAPI)
+    maintain_csv2platodb_A_uploadFile.uploadFolder(folderName,tgtGOfficialAPI,cookie=cookie)
+    
+    with open(folderName+"/uploadSchema.json","r") as uploadSchemaFile:
+        uploadSchemaJson=json.load(uploadSchemaFile)
+    gspace=uploadSchemaJson["gDbName"]
+    
+    maintain_csv2platodb_B_submitSchema.createSchemaFromSchemaJson(uploadSchemaJson,graphClient=gClient)
+    gClient.execute_query("USE {}".format(gspace))
+    gClient.set_space(gspace)
+    
+    vertexJsonList=[]
+    edgeJsonList=[]
+    if "vertex" in uploadSchemaJson:
+        vertexJsonList=maintain_csv2platodb_B_submitSchema.buildVertex(uploadSchemaJson["vertex"],graphClient=gClient)
+    if "edge" in uploadSchemaJson:
+        edgeJsonList=maintain_csv2platodb_B_submitSchema.buildEdge(uploadSchemaJson["edge"],graphClient=gClient)
+        
+    time.sleep(15) # 构造完成schema后延时15秒
+    
+    schemaJson={
+        "version": "v1rc1",
+        "description": "web console import",
+        "clientSettings": {
+            "concurrency": 10,
+            "channelBufferSize": 128,
+            "space": gspace,
+            "connection": {
+                "user": tgtGdbUser,
+                "password": tgtGdbPassword,
+                "address": "{}:{}".format(tgtGdbHost,tgtGdbPort)
+            }
+        },
+        "logPath": "/upload-dir/tmp/import.log",
+        "files": vertexJsonList+edgeJsonList
+    }
+    with open(folderName+"/schemaJson.json","w+") as schemaJsonFile:
+        json.dump(schemaJson,schemaJsonFile)
+        
+    taskId=maintain_csv2platodb_B_submitSchema.submitSchema(schemaJson,gUrl=tgtGOfficialAPI,cookie=cookie)
+    maintain_csv2platodb_C_import.importData(tgtGOfficialAPI,taskId=taskId,cookie=cookie)
+
+from pyecharts import options as opts
+from pyecharts.charts import Graph
+def showSchema(gClient,gSpace,tabuEdgeList=["Alias","School"],tabuTagList=["aliasNameOf","planOf"]):
+    '''显示图数据库schema'''
+    gClient.execute_query("USE {}".format(gSpace))
+    gClient.set_space(gSpace)
+    
+    tagIndexDf=wrapNebula2Df(gClient.execute_query("SHOW TAG INDEXES"))
+    tagIndexList=tagIndexDf["Index Name"].values.flatten().tolist()
+
+    tagDf=wrapNebula2Df(gClient.execute_query("SHOW TAGS"))
+    tagList=[]
+    tagCountDict={}
+    if tagDf.shape[0]>0:
+        tagList=tagDf["Name"].values.flatten().tolist()
+
+        tagAttrDict={}
+        tagCountDict={}
+        for tagIndexItem in tagIndexList:
+            if tagIndexItem not in tabuTagList:
+                tagIndexDf=wrapNebula2Df(gClient.execute_query("DESCRIBE TAG INDEX {}".format(tagIndexItem)))
+                tagAttrList=tagIndexDf["Field"].values.flatten().tolist()
+                for tagAttrItem in tagAttrList:
+                    for tagItem in tagList:
+                        tagSearchDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {tagType} WHERE {tagType}.{tagAttr}!='不可能的名字'|YIELD COUNT($-.VertexID) AS nodeCount".format(
+                                                                                                                                                        tagType=tagItem,
+                                                                                                                                                        tagAttr=tagAttrItem
+                                                                                                                                                    ))).replace("",np.nan).replace("null",np.nan).dropna()
+                        if tagSearchDf.shape[0]>0:
+                            tagCountVal=tagSearchDf["nodeCount"].values[0]
+                            if tagCountVal>0:
+                                tagAttrDict[tagItem]=tagAttrDict.get(tagItem,[])+[tagAttrItem]
+                                tagCountDict[tagItem]=tagCountVal
+    
+    edgeDf=wrapNebula2Df(gClient.execute_query("SHOW EDGES")).dropna()
+    if edgeDf.shape[0]>0:
+        edgeList=edgeDf["Name"].values.flatten().tolist()
+    
+    htList=[]
+    htSetList=[]
+    for edgeTypeItem in edgeList:
+        if edgeTypeItem not in tabuEdgeList:
+            for headTypeI in range(len(tagList)):
+                for tailTypeI in range(headTypeI,len(tagList)):
+                    headType=tagList[headTypeI]
+                    tailType=tagList[tailTypeI]
+                    htPairStr=str(sorted([headType,tailType]))
+                    if headType in tagAttrDict and tailType in tagAttrDict:
+                        for headAttr in tagAttrDict[headType]:
+                            for tailAttr in tagAttrDict[tailType]:
+                                if headType not in tabuTagList and tailType not in tabuTagList:
+                                    if htPairStr not in htSetList:
+                                        htDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {headType} WHERE {headType}.{headIdAttr}!='不可能的名字'|\
+                                                                                    LIMIT 5|\
+                                                                                    GO FROM $-.VertexID OVER {edgeType} BIDIRECT YIELD $$.{tailType}.{tailIdAttr}".format(
+                                                                                        headType=headType,
+                                                                                        headIdAttr=headAttr,
+                                                                                        edgeType=edgeTypeItem,
+                                                                                        tailType=tailType,
+                                                                                        tailIdAttr=tailAttr,
+                                                                                    ))).replace("",np.nan).replace("null",np.nan).dropna()
+                                        if htDf.shape[0]>0:
+                                            htList.append([headType,edgeTypeItem,tailType])
+                                            htSetList.append(htPairStr)
+    
+    return tagCountDict,htList
+
+
+
 def downloadGraph(gClient,
                     srcGdbName,srcFakeGdbName,
                     edgeTypeList=[],
@@ -464,9 +593,9 @@ def downloadGraph(gClient,
         os.mkdir("downloadGraph")
     rawSchema={"gDbName":srcFakeGdbName,"vertex":[],"edge":[]}
     gClient.execute_query("USE {}".format(srcGdbName))
-    uploadFolderName="tmp_{}".format(int(time.time()*1000))
-    if uploadFolderName not in os.listdir("./downloadGraph"):
-        os.mkdir("./downloadGraph/"+uploadFolderName)
+    downloadFolderName="tmp_{}".format(int(time.time()*1000))
+    if downloadFolderName not in os.listdir("./downloadGraph"):
+        os.mkdir("./downloadGraph/"+downloadFolderName)
     for srcVertexTypeItem in srcVertexKeynameDict:
         lookupVertexGroupStr="LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}!='不可能的名字'|FETCH PROP ON {nodeType} $-.VertexID".format(
                                                                                                                                                 nodeType=srcVertexTypeItem,
@@ -497,7 +626,7 @@ def downloadGraph(gClient,
                     vertexDfItem[colItem].fillna(0.0,inplace=True)
                     vertexDfItem[colItem]=vertexDfItem[colItem].astype(float)
         
-        vertexDfItem.to_csv("./downloadGraph/{}/{}.csv".format(uploadFolderName,srcVertexTypeItem),index=None)
+        vertexDfItem.to_csv("./downloadGraph/{}/{}.csv".format(downloadFolderName,srcVertexTypeItem),index=None)
         vertexSchemaItem={
                             "cover_label":"escape,cover",
                             "file_name":"{}.csv".format(srcVertexTypeItem),
@@ -507,78 +636,95 @@ def downloadGraph(gClient,
                             "attr_type_map":attrDTypeDict
                         }
         rawSchema["vertex"].append(vertexSchemaItem)
-        
+    
+    edgeBatchSize=1024
     for edgeTypeItem in edgeTypeList:
         for srcVertexTypeItem_head in srcVertexKeynameDict:
             for tgtVertexTypeItem_tail in srcVertexKeynameDict:
-                edgeLookupReq=gClient.execute_query("LOOKUP ON {srcNodeType} WHERE {srcNodeType}.{srcNodeIdAttr}!='不可能的名字'|\
-                                                        GO FROM $-.VertexID OVER {edgeType} YIELD   {edgeType}._src AS srcSysID,\
-                                                                                                    {edgeType}._dst AS tgtSysID,\
-                                                                                                    $^.{srcNodeType}.{srcNodeIdAttr} AS {srcNodeType}{srcNodeIdAttr}_head,\
-                                                                                                    $$.{tgtNodeType}.{tgtNodeIdAttr} AS {tgtNodeType}{tgtNodeIdAttr}_tail".format(
-                                                                                                        srcNodeType=srcVertexTypeItem_head,
-                                                                                                        srcNodeIdAttr=srcVertexKeynameDict[srcVertexTypeItem_head],
-                                                                                                        edgeType=edgeTypeItem,
-                                                                                                        tgtNodeType=tgtVertexTypeItem_tail,
-                                                                                                        tgtNodeIdAttr=srcVertexKeynameDict[tgtVertexTypeItem_tail]
-                                                                                                    ))
-                edgeLookupDf=wrapNebula2Df(edgeLookupReq).dropna()
-                
-                edgeSysIdList=edgeLookupDf.loc[:,["srcSysID","tgtSysID"]].astype(str).values.tolist()
-                edgeSysIdListStr=",".join(["->".join(edgeSysIdItem) for edgeSysIdItem in edgeSysIdList])
-                edgeFetchReq=gClient.execute_query("FETCH PROP ON {edgeType} {edgeList}".format(
-                                                                                                    edgeType=edgeTypeItem,
-                                                                                                    edgeList=edgeSysIdListStr
-                                                                                                ))
-                edgeFetchDf=wrapNebula2Df(edgeFetchReq)
-                
-                edgeDfItem=pd.merge(edgeLookupDf,edgeFetchDf,
-                                    left_on=["srcSysID","tgtSysID"],
-                                    right_on=["{edgeType}._src".format(edgeType=edgeTypeItem),
-                                              "{edgeType}._dst".format(edgeType=edgeTypeItem)])
-                renameColDict=dict((colItem,colItem.split(".")[1])
-                                    if "." in colItem else (colItem,colItem)
-                                    for colItem in edgeDfItem.columns)
-                edgeDfItem.rename(renameColDict,axis=1,inplace=True)
-                
-                edgeAttrDTypeDf=wrapNebula2Df(gClient.execute_query("DESCRIBE EDGE {}".format(edgeTypeItem)))
-                edgeAttrList=edgeAttrDTypeDf["Field"].values.flatten().tolist()
-                # csv2platodb
-                csv2platoAttrDict=dict((colItem,colItem) for colItem in vertexDfItem.columns  if colItem in edgeAttrList)
-                
-                # platoAttrType
-                attrDTypeDict=dict(edgeAttrDTypeDf.values.tolist())
-                for colItem in edgeDfItem.columns:
-                    if colItem in attrDTypeDict:
-                        if attrDTypeDict[colItem]=="string":
-                            edgeDfItem[colItem].fillna("null",inplace=True)
-                            edgeDfItem[colItem]=edgeDfItem[colItem].astype(str)
-                        elif attrDTypeDict[colItem]=="int":
-                            edgeDfItem[colItem].fillna(0,inplace=True)
-                            edgeDfItem[colItem]=edgeDfItem[colItem].astype(int)
-                        elif attrDTypeDict[colItem]=="double":
-                            edgeDfItem[colItem].fillna(0.0,inplace=True)
-                            edgeDfItem[colItem]=edgeDfItem[colItem].astype(float)
-                
-                edgeDfItem.to_csv("./downloadGraph/{}/{}.csv".format(uploadFolderName,edgeTypeItem),index=None)
-                edgeSchemaItem={
-                    "cover_label":"escape,escape,cover",
-                    "file_name":"{}.csv".format(edgeTypeItem),
-                    "edge_type":edgeTypeItem,
-                    "src_type":srcVertexTypeItem_head,
-                    "tgt_type":tgtVertexTypeItem_tail,
-                    "src_id":"{srcNodeType}{srcNodeIdAttr}_head".format(srcNodeType=srcVertexTypeItem_head,srcNodeIdAttr=srcVertexKeynameDict[srcVertexTypeItem_head]),
-                    "tgt_id":"{tgtNodeType}{tgtNodeIdAttr}_tail".format(tgtNodeType=tgtVertexTypeItem_tail,tgtNodeIdAttr=srcVertexKeynameDict[tgtVertexTypeItem_tail]),
-                    "csv2plato_attr_map":csv2platoAttrDict,
-                    "attr_type_map":attrDTypeDict
-                }
-                
-                rawSchema["edge"].append(edgeSchemaItem)
+                edgeBatchI=0
+                while True:
+                    try:
+                        edgeLookupReq=gClient.execute_query("LOOKUP ON {srcNodeType} WHERE {srcNodeType}.{srcNodeIdAttr}!='不可能的名字'|\
+                                                                GO FROM $-.VertexID OVER {edgeType} YIELD   {edgeType}._src AS srcSysID,\
+                                                                                                            {edgeType}._dst AS tgtSysID,\
+                                                                                                            $^.{srcNodeType}.{srcNodeIdAttr} AS {srcNodeType}{srcNodeIdAttr}_head,\
+                                                                                                            $$.{tgtNodeType}.{tgtNodeIdAttr} AS {tgtNodeType}{tgtNodeIdAttr}_tail|\
+                                                                                                                LIMIT {batchI},{batchSize}".format(
+                                                                                                                srcNodeType=srcVertexTypeItem_head,
+                                                                                                                srcNodeIdAttr=srcVertexKeynameDict[srcVertexTypeItem_head],
+                                                                                                                edgeType=edgeTypeItem,
+                                                                                                                tgtNodeType=tgtVertexTypeItem_tail,
+                                                                                                                tgtNodeIdAttr=srcVertexKeynameDict[tgtVertexTypeItem_tail],
+                                                                                                                batchI=edgeBatchI,
+                                                                                                                batchSize=edgeBatchSize
+                                                                                                            ))
+                        edgeLookupDf=wrapNebula2Df(edgeLookupReq).dropna()
+                        if edgeLookupDf.shape[0]==0:
+                            break
+                        if edgeLookupDf.shape[0]>0:
+                            edgeSysIdList=edgeLookupDf.loc[:,["srcSysID","tgtSysID"]].astype(str).values.tolist()
+                            edgeSysIdListStr=",".join(["->".join(edgeSysIdItem) for edgeSysIdItem in edgeSysIdList])
+                            edgeFetchReq=gClient.execute_query("FETCH PROP ON {edgeType} {edgeList}".format(
+                                                                                                                edgeType=edgeTypeItem,
+                                                                                                                edgeList=edgeSysIdListStr
+                                                                                                            ))
+                            edgeFetchDfItem=wrapNebula2Df(edgeFetchReq)
+                            
+                            edgeDfItem=pd.merge(edgeLookupDf,edgeFetchDfItem,
+                                                left_on=["srcSysID","tgtSysID"],
+                                                right_on=["{edgeType}._src".format(edgeType=edgeTypeItem),
+                                                        "{edgeType}._dst".format(edgeType=edgeTypeItem)],
+                                                how="right")
+                            renameColDict=dict((colItem,colItem.split(".")[1])
+                                                if "." in colItem else (colItem,colItem)
+                                                for colItem in edgeDfItem.columns)
+                            edgeDfItem.rename(renameColDict,axis=1,inplace=True)
+                            
+                            edgeAttrDTypeDf=wrapNebula2Df(gClient.execute_query("DESCRIBE EDGE {}".format(edgeTypeItem)))
+                            edgeAttrList=edgeAttrDTypeDf["Field"].values.flatten().tolist()
+                            # csv2platodb
+                            csv2platoAttrDict=dict((colItem,colItem) for colItem in vertexDfItem.columns  if colItem in edgeAttrList)
+                            
+                            # platoAttrType
+                            attrDTypeDict=dict(edgeAttrDTypeDf.values.tolist())
+                            for colItem in edgeDfItem.columns:
+                                if colItem in attrDTypeDict:
+                                    if attrDTypeDict[colItem]=="string":
+                                        edgeDfItem[colItem].fillna("null",inplace=True)
+                                        edgeDfItem[colItem]=edgeDfItem[colItem].astype(str)
+                                    elif attrDTypeDict[colItem]=="int":
+                                        edgeDfItem[colItem].fillna(0,inplace=True)
+                                        edgeDfItem[colItem]=edgeDfItem[colItem].astype(int)
+                                    elif attrDTypeDict[colItem]=="double":
+                                        edgeDfItem[colItem].fillna(0.0,inplace=True)
+                                        edgeDfItem[colItem]=edgeDfItem[colItem].astype(float)
+                            
+                            edgeDfItem.to_csv("./downloadGraph/{}/{}_{}_{}.csv".format(downloadFolderName,edgeTypeItem,
+                                                                                        edgeBatchI,edgeBatchI+edgeBatchSize),index=None)
+                            edgeSchemaItem={
+                                "cover_label":"escape,escape,cover",
+                                "file_name":"{}_{}_{}.csv".format(downloadFolderName,edgeTypeItem,
+                                                                                        edgeBatchI,edgeBatchI+edgeBatchSize),
+                                "edge_type":edgeTypeItem,
+                                "src_type":srcVertexTypeItem_head,
+                                "tgt_type":tgtVertexTypeItem_tail,
+                                "src_id":"{srcNodeType}{srcNodeIdAttr}_head".format(srcNodeType=srcVertexTypeItem_head,srcNodeIdAttr=srcVertexKeynameDict[srcVertexTypeItem_head]),
+                                "tgt_id":"{tgtNodeType}{tgtNodeIdAttr}_tail".format(tgtNodeType=tgtVertexTypeItem_tail,tgtNodeIdAttr=srcVertexKeynameDict[tgtVertexTypeItem_tail]),
+                                "csv2plato_attr_map":csv2platoAttrDict,
+                                "attr_type_map":attrDTypeDict
+                            }
+                            
+                            rawSchema["edge"].append(edgeSchemaItem)
+                    except:
+                        edgeBatchI-=edgeBatchSize
+                    print("edge batch:",edgeBatchI)
+                    edgeBatchI+=edgeBatchSize
                 
     
-    with open("./downloadGraph/{}/rawSchema.json".format(uploadFolderName),"w+") as rawSchemaFile:
+    with open("./downloadGraph/{}/rawSchema.json".format(downloadFolderName),"w+") as rawSchemaFile:
         json.dump(rawSchema,rawSchemaFile)
         
+    print("文件已保存于{}".format(downloadFolderName))
 
 
 if __name__=="__main__":
@@ -588,7 +734,7 @@ if __name__=="__main__":
     srcGUser="root"
     srcGPassword="nebula"
     srcGdbName="post_dept_char_major_skill__wliangwang"
-    
+
     srcConnection_pool = ConnectionPool(srcGHost, srcGPort,network_timeout=300000)
     srcClient = GraphClient(srcConnection_pool)
     srcClient.authenticate(srcGUser, srcGPassword)
@@ -642,7 +788,15 @@ if __name__=="__main__":
     #                      csv2platoDTypeDict=csv2platoDTypeDict,
     #                      projectName=projectName)
     
-    downloadGraph(srcClient,"post_skill_school_ianxu","post_tag",
-                  edgeTypeList=["subPost"],
-                  srcVertexKeynameDict={"YunPost":"YunPostName"}
-                  )
+    # downloadGraph(srcClient,"post_skill_school_ianxu","post_tag",
+    #               edgeTypeList=["subPost"],
+    #               srcVertexKeynameDict={"YunPost":"YunPostName"}
+    #               )
+
+    start=time.time()
+    tagList,htList=showSchema(srcClient,"post_skill_school_ianxu")
+    end=time.time()
+
+    print(end-start)
+
+    print(123)
\ No newline at end of file
